<?php
namespace PTPay\MeoWallet\SDK\Resources;

use PTPay\MeoWallet\SDK\Exceptions\ApiResponseException;

/**
 * Class Checkout
 * @package PTPay\MeoWallet\SDK\Resources
 * @see https://developers.wallet.pt/en/procheckout/resources.html#api_v2_checkout
 */
class Checkout extends ApiResource
{

    const METHOD_WALLET = 'WALLET';
    const METHOD_CC = 'CC';
    const METHOD_MB = 'MB';
    const METHOD_PAYPAL = 'PAYPAL';
    const METHOD_MBWAY = 'MBWAY';




    /**
     * Checkout constructor.
     * @param \PTPay\MeoWallet\SDK\Api $api
     */
    public function __construct($api)
    {
        parent::__construct($api);
    }

    /**
     * Create a checkout
     * @param \PTPay\MeoWallet\SDK\Structures\Client $client
     * @param array $items
     * @param float $amount
     * @param string $currency
     * @param string $urlConfirm
     * @param string $urlCancel
     * @param null|int $extCustomerId
     * @param null|int $extInvoiceId
     * @param null|string $extEmail
     * @param null|string $defaultMethod
     * @return array
     */
    public function create(
        $client,
        $items,
        $amount,
        $currency,
        $urlConfirm,
        $urlCancel,
        $extCustomerId = null,
        $extInvoiceId = null,
        $extEmail = null,
        $defaultMethod = null
    ) {
        $requestClient = [
            'name'    => $client->getName(),
            'email'   => $client->getEmail(),
            'address' => [
                'country'    => $client->getAddress()->getCountry(),
                'address'    => $client->getAddress()->getAddress(),
                'city'       => $client->getAddress()->getCity(),
                'postalcode' => $client->getAddress()->getPostalcode(),
            ]
        ];
        $requestPayment = [
            'client'   => $requestClient,
            'amount'   => round($amount,2),
            'currency' => $currency,
        ];
        if ($extInvoiceId) {
            $requestPayment['ext_invoiceid'] = $extInvoiceId;
        }
        if ($extCustomerId) {
            $requestPayment['ext_customerid'] = $extCustomerId;
        }
        if ($extEmail) {
            $requestPayment['ext_email'] = $extEmail;
        }
        $requestItems = [];
        foreach ($items as $item) {
            /** @var \PTPay\MeoWallet\SDK\Structures\Item $item */
            $requestItems[] = [
                'ref'   => $item->getRef(),
                'name'  => $item->getName(),
                'descr' => $item->getDescription(),
                'qt'    => $item->getQuantity(),
            ];
        }
        $requestPayment['items'] = $requestItems;


        switch ($defaultMethod) {
            case 'WALLET':
            case 'CC':
            case 'MB':
            case 'PAYPAL':
            case 'MBWAY':
                $method = $defaultMethod;
                break;
            default:
                $method = "WALLET";
                break;
        }

        $requestData = [
            'payment'     => $requestPayment,
            'url_confirm' => $urlConfirm,
            'url_cancel'  => $urlCancel,
            'default_method' => $method,

        ];


        $response = $this->api->call('POST', '/checkout', $requestData);

        return $response->getDataAsArray();
    }

    /**
     * Get checkout data
     * @param string $checkoutId
     * @return array
     */
    public function get($checkoutId)
    {
        return $this->api->call('GET', "/checkout/{$checkoutId}")->getDataAsArray();
    }

    /**
     * Delete a checkout
     * @param string $checkoutId
     * @return mixed
     */
    public function delete($checkoutId)
    {
        return $this->api->call('DELETE', "/checkout/{$checkoutId}")->getDataAsArray();
    }

    /**
     * Verify if the callback is genuine
     *
     * @param array $callbackData Callback data
     * @deprecated
     * @return bool
     */
    public function verify($callbackData)
    {
       return $this->isValidCallback($callbackData);
    }
    /**

     * Verify if the callback is valid
     *
     * @param array|string $data Array or json string
     *
     * @return bool
     *
     */
    public function isValidCallback($data) {
        if (is_object($data)) {
            $data = get_object_vars($data);
        }
        $data = is_array($data) ? $data : json_decode($data,true);
        try {
            $response = $this->api->call('POST', "/callback/verify", $data);
            //return $response->getHttpCode() === 200;
		return true;
        } catch (ApiResponseException $e){
            return false;
        }
    }

    /**
     * @deprecated checkout page now identifies user locale
     * @param $locale
     * @return mixed
     */

    private function normalizeLocale($locale) {
        $supported = ['pt_pt',"en_us","fr_fr"];
        foreach($supported as $code) {
            if (stripos($code,$locale) !== false) {
                return $code;
            }
        }
        return $supported[0];
    }
    public static function getSupportedMethods() {
        return [static::METHOD_WALLET,static::METHOD_CC,static::METHOD_MB,static::METHOD_PAYPAL,static::METHOD_MBWAY];
    }

}
